import { SoundcraftUI } from 'soundcraft-ui-connection';
import $ from 'jquery';
import './index.css';

const HOST = '192.168.0.111';
const soundcraft = new SoundcraftUI(HOST);
const app = $('#app');

// https://github.com/fmalcher/soundcraft-ui/tree/main/packages/mixer-connection

soundcraft.connect();
console.log(soundcraft);

const channels = [
  {
    name: 'mikrid 1 ja 2',
    channel: [1, 2],
    separate: true,
  },
  {
    name: 'lava',
    channel: [5],
  },
  {
    name: 'hdmi',
    channel: [6],
  },
];

const faders = [
  {
    channel: [5, 6],
    separate: true
  }
]

soundcraft.status$.subscribe((status) => {
  console.log(status);
  if (status.type !== 'OPEN') {
    return;
  }
  app.empty();
  channels.forEach((channel) => {
    const managers = channel.channel.map((num) => soundcraft.master.input(num));
    const statuses = channel.channel.map(() => false);
    const wrapper = $('<div>').addClass('channel');
    const statusText = $('<span>').addClass('status');
    const btnGroup = $('<div>').addClass('btn-group');
    const mute = $('<button>').text('välja');
    const unmute = $('<button>').text('sisse');
    wrapper.append($('<span>').text(channel.name).addClass('title'));
    wrapper.append(statusText);
    if (channel.separate) {
      wrapper.addClass('separated');
      channel.channel.forEach((num, index) => {
        const ibtnGroup = $('<div>').addClass('btn-group');
        const imute = $('<button>').text(`${index + 1} välja`);
        const iunmute = $('<button>').text(`${index + 1} sisse`);
        imute.on('click', () => managers[index].mute());
        iunmute.on('click', () => managers[index].unmute());
        ibtnGroup.append(imute, iunmute);
        wrapper.append(ibtnGroup);
      });
      mute.text('mõlemad välja');
      unmute.text('mõlemad sisse');
    }
    channel.channel.forEach((num, index) => {
      managers[index].mute$.subscribe((status) => {
        statuses[index] = status === 1;
        let text = '';
        if (channel.separate) {
          if (statuses.every((stat) => stat === true)) {
            text = 'kõik on väljas';
          } else {
            const off = statuses.map(
              (stat, index) => stat ? (index + 1) : false
            ).filter((stat) => stat !== false);
            if (!off.length) {
              text = 'kõik on sees';
            } else {
              text = off.join(', ') + ' on väljas';
            }
          }
        } else {
          text = status ? 'väljas' : 'sees';
        }
        statusText.text(text);
      });
    });
    mute.on('click', () => managers.forEach((manager) => manager.mute()));
    unmute.on('click', () => managers.forEach((manager) => manager.unmute()));
    btnGroup.append(mute, unmute);
    wrapper.append(btnGroup);
    app.append(wrapper);
  });

  faders.forEach((channel) => {
    const managers = channel.channel.map((num) => soundcraft.master.input(num));
    const wrapper = $('<div>').addClass('fader');
    channel.channel.forEach((num, index) => {
      const statusText = $('<span>').addClass('volume');
      const input = $('<input>').attr('min', 0).attr('max', 1).attr('step', 0.01).attr('type', 'range');
      input.on('input', () => {
        managers[index].setFaderLevel(input.val() as number);
      });
      managers[index].faderLevel$.subscribe((level) => {
        statusText.text(`${Math.floor(level * 100)}%`);
        input.val(level);
      });
      wrapper.append(statusText);
      wrapper.append(input);
    });
    app.append(wrapper);
  });
});
